package dbschemaextra

import (
	_ "embed"
	"git.sr.ht/~mil/mobsql"
)

//go:embed sqlviews/_vstopparent.sql
var _vstopparent string

//go:embed sqlviews/_vconnd.sql
var _vconnd string

//go:embed sqlviews/_vconndverbose.sql
var _vconndverbose string

//go:embed sqlviews/_vcaltoservice.sql
var _vcaltoservice string

//go:embed sqlviews/_vtransferstbl.sql
var _vtransferstbl string

//go:embed sqlviews/_vtransfersgen.sql
var _vtransfersgen string

//go:embed sqlviews/_vtransfersimplicit.sql
var _vtransfersimplicit string

//go:embed pragmas.sql
var pragmas string

var MobsqlSchemaExtra = mobsql.SchemaExtra{
	InitExec: pragmas,
	Views: []mobsql.View{
		{Name: "_vstopparent", Doc: "Mapping of stops to their parent stops", SQLCreateStatement: _vstopparent},
		{Name: "_vconnd", Doc: "CSA timetable", SQLCreateStatement: _vconnd},
		{Name: "_vconndverbose", Doc: "CSA timetable (with verbose joins)", SQLCreateStatement: _vconndverbose},
		{Name: "_vcaltoservice", Doc: "Maps calendar dates individually to valid service id", SQLCreateStatement: _vcaltoservice},
		{Name: "_vtransferstbl", Doc: "Transfers table added with from/to idu (parents)", SQLCreateStatement: _vtransferstbl},
		{Name: "_vtransfersgen", Doc: "Transfers table generated from stops self-join", SQLCreateStatement: _vtransfersgen},
		{Name: "_vtransfersimplicit ", Doc: "Implicit transfers ", SQLCreateStatement: _vtransfersimplicit},
	},
	TablesComputed: []mobsql.ComputedTable{
		{

			Doc:           `Computed table for the _cvconnd view`,
			View:          `_vconnd`,
			CacheOnTables: []string{"stop_times", "trips", "stops", "routes"},
			TableSpec: mobsql.TableSpec{
				Table:     "_cvconnd",
				Indexes:   [][]string{{"source", "service_id", "from_stop_time"}},
				UniqueSet: []string{"source", "conn_id"},
				Columns: []mobsql.LoadColumn{
					{Column: "source", DType: mobsql.DTypeInt, Required: true},
					{Column: "conn_id", DType: mobsql.DTypeText, Required: true},
					{Column: "trip_id", DType: mobsql.DTypeText, Required: true},
					{Column: "service_id", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_id", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_idu", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_sidu", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_time", DType: mobsql.DTypeInt, Required: true},
					{Column: "from_stop_sequence", DType: mobsql.DTypeInt, Required: true},
					{Column: "to_stop_id", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_idu", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_sidu", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_time", DType: mobsql.DTypeInt, Required: true},
					{Column: "to_stop_sequence", DType: mobsql.DTypeInt, Required: true},
				},
			},
		},
		{
			View:          "_vcaltoservice",
			SourcePair:    false,
			CacheOnTables: []string{"stops"},
			TableSpec: mobsql.TableSpec{
				Table: "_cvcaltoservice",
				Indexes: [][]string{
					{"service_date", "source"},
				},
				Columns: []mobsql.LoadColumn{
					{Column: "source", DType: mobsql.DTypeInt, Required: true},
					{Column: "service_date", DType: mobsql.DTypeInt, Required: true},
					{Column: "service_id", DType: mobsql.DTypeText, Required: true},
				},
			},
		},
		{
			View:          "_vtransfersgen",
			SourcePair:    true,
			CacheOnTables: []string{"stops"},
			TableSpec: mobsql.TableSpec{
				Table: "_cvtransfersgen",
				Columns: []mobsql.LoadColumn{
					{Column: "source", DType: mobsql.DTypeInt, Required: true},
					{Column: "source_pair", DType: mobsql.DTypeInt, Required: true},
					{Column: "from_stop_id", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_id", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_idu", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_idu", DType: mobsql.DTypeText, Required: true},
					{Column: "from_stop_sidu", DType: mobsql.DTypeText, Required: true},
					{Column: "to_stop_sidu", DType: mobsql.DTypeText, Required: true},
					{Column: "dist_km", DType: mobsql.DTypeReal},
				},
			},
		},
	},
}
