-- Currently set for 3km max distance for generated transfers
with rough_stop_bounds as (
  -- Coarse bounds check to avoid full haversine dist stops self-join
  select
    source,
    rowid as rid,
    stop_id,
    stop_lat,
    stop_lon,
    stop_lat - (3.0 / 111.0) as min_lat,
    stop_lat + (3.0 / 111.0) as max_lat,
    stop_lon - (3.0 / (111.0 * cos(radians(stop_lat)))) as min_lon,
    stop_lon + (3.0 / (111.0 * cos(radians(stop_lat)))) as max_lon
  from
    stops
)
select
  s.source,
  sb.source as source_pair,
  s.stop_id as from_stop_id,
  sb.stop_id as to_stop_id,
  v1.parent_station as from_stop_idu,
  v2.parent_station as to_stop_idu,
  v1.source || '_' || v1.parent_station as from_stop_sidu,
  v2.source || '_' || v2.parent_station as to_stop_sidu,
  6371.0 * acos(
    sin(90.0 - sb.stop_lat * pi() / 180.0) *
    sin(90.0 - s.stop_lat * pi() / 180.0) *
    cos(
      (sb.stop_lon * pi() / 180.0) -
      (s.stop_lon * pi() / 180.0)
    ) +
    cos(90.0 - sb.stop_lat * pi() / 180.0) *
    cos(90.0 - s.stop_lat * pi() / 180.0)
  ) as dist_km
from rough_stop_bounds sb
  join stops s
    on (s.stop_lat between sb.min_lat and sb.max_lat)
    and (s.stop_lon between sb.min_lon and sb.max_lon)
    and (s.source < sb.source or (s.source = sb.source and sb.rid < s.rowid))
    and (dist_km < 3.0)
  join _vstopparent v1 on s.source = v1.source and s.stop_id = v1.stop_id and v1.stop_id = v1.parent_station
  join _vstopparent v2 on sb.source = v2.source and sb.stop_id = v2.stop_id and v2.stop_id = v2.parent_station
