package apiroute

import (
	"git.sr.ht/~mil/mobroute/api/apiroute/diagnosticst"
	"git.sr.ht/~mil/mobroute/api/apiroute/formatterdiagnostics"
	"git.sr.ht/~mil/mobroute/api/apiroute/formattergeojson"
	"git.sr.ht/~mil/mobroute/api/apiroute/formatterlegs"
	"git.sr.ht/~mil/mobroute/api/apiroute/formattermapurl"
	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
)

func formatRouteResponse(
	diagnostics *diagnosticst.Diagnostics,
	connectionsVerbose *[]dbt.ConnectionVerbose,
	transfers *[]dbt.Transfer,
	routeParams *RouteParams,
) *RouteResponse {
	var (
		r                 = RouteResponse{}
		formatConnections = utilfuncs.ItemInArray("connections", *routeParams.OutputFormats)
		formatLegs        = utilfuncs.ItemInArray("legs", *routeParams.OutputFormats)
		formatGeojson     = utilfuncs.ItemInArray("geojson", *routeParams.OutputFormats)
		formatMapurl      = utilfuncs.ItemInArray("mapurl", *routeParams.OutputFormats)
		formatDiagnostics = utilfuncs.ItemInArray("diagnostics", *routeParams.OutputFormats)
		formatRequest     = utilfuncs.ItemInArray("request", *routeParams.OutputFormats)
	)

	// Formatters which depend / are derived from the base 'legs' format
	if formatLegs || formatGeojson || formatMapurl || formatDiagnostics {
		routeLegs := formatterlegs.FormatLegs(
			connectionsVerbose,
			transfers,
			routeParams.Time,
			*routeParams.WalkspeedKmHr,
			*routeParams.From,
			*routeParams.To,
		)
		if formatLegs {
			r.RouteLegs = routeLegs
		}
		if formatGeojson {
			r.GeoJSON = formattergeojson.FormatGeoJSON(routeLegs)
		}
		if formatMapurl {
			r.MapURL = ptr(formattermapurl.FormatMapURL(formattergeojson.FormatGeoJSON(routeLegs)))
		}
		if formatDiagnostics {
			r.Diagnostics = formatterdiagnostics.FormatDiagnosticsExtra(diagnostics, routeLegs)
		}
	}

	// Raw connections dump
	if formatConnections {
		r.Connections = connectionsVerbose
	}

	// Raw request dump
	if formatRequest {
		r.Request = routeParams
	}

	return &r
}
