package formatterlegs

import (
	"fmt"

	"git.sr.ht/~mil/mobroute/dbt"
)

func FormatConnectionsToLegTrip(
	yyyymmdd uint,
	timezoneUTCDeltasecs int,
	legConnections []*dbt.ConnectionVerbose,
) *RouteLeg {
	var (
		connectionFrom = legConnections[0]
		connectionTo   = legConnections[len(legConnections)-1]
		timeFrom       = createTime(yyyymmdd, timezoneUTCDeltasecs, connectionFrom.FromStopDRUTCTime)
		timeTo         = createTime(yyyymmdd, timezoneUTCDeltasecs, connectionTo.ToStopDRUTCTime)
	)
	return &RouteLeg{
		LegType:           "trip",
		LegDuration:       timeTo.Sub(timeFrom).String(),
		LegFromCoords:     fmt.Sprintf("%0.04f, %0.04f", connectionFrom.FromStopLat, connectionFrom.FromStopLon),
		LegToCoords:       fmt.Sprintf("%0.04f, %0.04f", connectionTo.ToStopLat, connectionTo.ToStopLon),
		LegBeginTime:      timeFrom,
		LegEndTime:        timeTo,
		TripRouteHeadsign: connectionFrom.TripHeadsign,
		TripRoute:         coalesce(connectionFrom.RouteShortName, connectionFrom.RouteLongName),
		TripFrom:          &connectionFrom.FromStopName,
		TripFromSIDU:      &connectionFrom.FromStopSIDU,
		TripTo:            &connectionTo.ToStopName,
		TripToSIDU:        &connectionTo.ToStopSIDU,
		TripAgencyID:      connectionTo.AgencyID,
		TripSource:        &connectionTo.Source,
		TripStops:         ptr(FormatLegsStopDetails(yyyymmdd, timezoneUTCDeltasecs, legConnections)),
	}
}

func FormatLegsStopDetails(
	yyyymmdd uint,
	timezoneUTCDeltasecs int,
	legConnections []*dbt.ConnectionVerbose,
) []*StopDetails {
	m := []*StopDetails{}
	for j, c := range legConnections {
		m = append(m, &StopDetails{
			Time:   createTime(yyyymmdd, timezoneUTCDeltasecs, c.FromStopDRUTCTime),
			Name:   c.FromStopName,
			Coords: fmt.Sprintf("%0.04f, %0.04f", c.FromStopLat, c.FromStopLon),
			ConnID: c.ConnID,
		})
		if j == len(legConnections)-1 {
			m = append(m, &StopDetails{
				Time:   createTime(yyyymmdd, timezoneUTCDeltasecs, c.ToStopDRUTCTime),
				Name:   c.ToStopName,
				Coords: fmt.Sprintf("%0.04f, %0.04f", c.ToStopLat, c.ToStopLon),
				ConnID: c.ConnID,
			})
		}
	}
	return m
}
