package dbquery

import (
	"fmt"
	"strings"

	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"github.com/jmoiron/sqlx"
)

// LoadConnectionsVerbose loads the connections for the given connection IDs
// note, that conn_id incoporates the source id as a prefix, so passing
// query sources here is solely for
func LoadConnectionsVerbose(
	DB *sqlx.DB, querySources []int, dateYYYYMMDD uint, fromSeconds, toSeconds int,
	connIDs []string,
) (*[]dbt.ConnectionVerbose, error) {
	var (
		r      = []dbt.ConnectionVerbose{}
		params = append(
			[]any{dateYYYYMMDD, fromSeconds, toSeconds},
			utilfuncs.Anyify(querySources)...,
		)
		query string
		err   error
	)

	if query, err = utilfuncs.TemplateExecute(
		query_connections,
		map[string]any{
			"Table":              "_vconndverbose",
			"Columns":            "*",
			"SourcePlaceholders": strings.Join(utilfuncs.ArrayWithValue(len(querySources), "?"), ","),
		},
	); err != nil {
		return nil, err
	}

	// Limit to only passed conn_id's
	query = fmt.Sprintf(
		"%s where conn_id in (%s)",
		query, strings.Join(utilfuncs.ArrayWithValue(len(connIDs), "?"), ","),
	)
	params = append(params, utilfuncs.Anyify(connIDs)...)

	if err := queryToSlice(DB, query, params, &r); err != nil {
		return nil, err
	} else {
		return &r, nil
	}
}
