package dbquery

import (
	"fmt"
	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"github.com/jmoiron/sqlx"
	"strings"
)

// LoadStopWalks loads stopwalks given the input origin coordinate and a max
// distance in km to walk
func LoadStopWalks(
	DB *sqlx.DB,
	querySources []int,
	originPoint utilfuncs.Coord,
	maxWalkKM float64,
) (*[]dbt.StopWalk, error) {
	var (
		r     = []dbt.StopWalk{}
		query = fmt.Sprintf(
			query_stopwalks,
			strings.Join(utilfuncs.ArrayWithValue(len(querySources), "?"), ","),
		)
		params = append(
			[]any{originPoint.Lat, originPoint.Lon, maxWalkKM},
			utilfuncs.Anyify(utilfuncs.ArrayIntToArryString(querySources))...,
		)
	)
	if err := queryToSlice(DB, query, params, &r); err != nil {
		return nil, err
	} else {
		return &r, nil
	}
}
