package dbquery

import (
	"fmt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"github.com/jmoiron/sqlx"
	"strconv"
	"strings"
)

// QueryBoundsMatch is a querying function that determines if the passed
// param matches for the mdbid for the DB conneciton min/max lat/lon
func QueryBoundsMatch(
	DB *sqlx.DB,
	querySources []int,
	coord utilfuncs.Coord,
) error {
	var (
		query = fmt.Sprintf(
			`
				select count(*) from (
					select 
						(? between min_lat and max_lat) and
						(? between min_lon and max_lon) as matches_bounds
					from _vsources
					where mdbid in (%s) and matches_bounds
				) g
			`,
			strings.Join(utilfuncs.ArrayWithValue(len(querySources), "?"), ","),
		)
		params = append(
			[]any{coord.Lat, coord.Lon},
			utilfuncs.Anyify(utilfuncs.ArrayIntToArryString(querySources))...,
		)
	)

	if value, err := QueryValue(DB, query, params); err != nil {
		return err
	} else if value == "" {
		return fmt.Errorf("%v doesn't match any of the provided bounds", coord)
	} else if matches, err := strconv.Atoi(value); err != nil {
		return err
	} else if matches < 1 {
		return fmt.Errorf("%v doesn't match any of the provided bounds - matches: %d, check bounds for sources: %v", coord, matches, querySources)
	} else {
		return nil
	}
}
