with
	inputs as (select ? as yyyymmdddate, ? as seconds_from, ? as seconds_to),
	sourcetotzutcdelta as (
		select source, ext_timezonetoutcdeltasecs(inputs.yyyymmdddate, min(agency_timezone)) as tzutcdeltasecs
		from agency join inputs on 1 = 1 group by source
	),
	timetable as (
		select
			cts.service_date,
			-- E.g. normalize from/to stop times in two ways:
			-- 1) normalize to the input date YYYYMMDD, so if a time departs at 02:00 on monday the following day;
			--    converted look like 26:00 on the previous day (since all times relative to input YYYYMMDD)
			-- 2) add the agency's associated timezone UTC delta value so that routing across agencies
			--    using different timezones functions correctly. All outuput values thus in UTC time
			--
			(cts.service_date - inputs.yyyymmdddate) * (60 * 60 * 24) + con.from_stop_time + tz.tzutcdeltasecs as from_stop_drutctime,
			(cts.service_date - inputs.yyyymmdddate) * (60 * 60 * 24) + con.to_stop_time + tz.tzutcdeltasecs as to_stop_drutctime,
			con.*
		from _cvcaltoservice cts
			join inputs on (
				-- E.g. optimization so we don't have to calculate {from,to}_stop_drutctime for *all*
				-- timetable entries (should handle both tz shifts & next day for schedule logic)
				cts.service_date between inputs.yyyymmdddate - 1 and inputs.yyyymmdddate + 1
			) and cts.source in ({{ .SourcePlaceholders }})
			join {{ .Table }} con
				on cts.service_id = con.service_id and cts.source = con.source
				and from_stop_drutctime between seconds_from and seconds_to
			join sourcetotzutcdelta tz on con.source = tz.source
			order by from_stop_drutctime asc
	)
	select {{ .Columns }} from timetable
