package dbquerytesthelper

import (
	"fmt"
	"os"

	"git.sr.ht/~mil/mobroute/api/apims"
	"git.sr.ht/~mil/mobroute/testhelper"
	"github.com/jmoiron/sqlx"
)

func PrepareMockLoadComputeAndOpenDB(
	mapSourceToYML map[int]string,
) (*sqlx.DB, string, error) {
	if mockMobilityDBCSV, gtfsZips, err := testhelper.PrepareMockMobilityDBMockFromGTFSYMLs(mapSourceToYML); err != nil {
		return nil, "", fmt.Errorf("Failed to prepare mock: %v", err)
	} else if sqliteDB, err := os.CreateTemp("", "mobroute_test_sqlite_*.db"); err != nil {
		return nil, "", fmt.Errorf("Failed to create mock SQLite DB: %v", err)
	} else if sqlxDB, _, err := apims.MSLoadComputeOpenRODB(
		&apims.MSParams{
			LoadFilter: nil,
			LoadConfig: &apims.RuntimeConfig{
				MobilityDBCatalogCSVURI: fmt.Sprintf("file://%s", mockMobilityDBCSV),
				SQLiteDBPath:            sqliteDB.Name(),
				AllowFileURIs:           true,
				LogInfo:                 true,
				LogWarn:                 true,
				LogDebug:                true,
			},
		},
	); err != nil {
		return nil, "", fmt.Errorf("Error: %v: DB: %s", err, sqliteDB.Name())
	} else {
		os.Remove(mockMobilityDBCSV)
		for _, z := range gtfsZips {
			os.Remove(z)
		}
		return sqlxDB, sqliteDB.Name(), nil
	}
}
