package utilfuncs

import (
	"fmt"
	"time"
)

func DurationFormatSecs(seconds uint) string {
	if seconds < 60 {
		return fmt.Sprintf("%ds", seconds)
	} else {
		minutes := seconds / 60
		secsrem := seconds % 60
		return fmt.Sprintf("%dm %ds", minutes, secsrem)
	}
}

// TODO: is this better to do via time package (AM/PM formatting free then?)
func TimeHMSFormatSecs(seconds uint) string {
	var (
		s = seconds % 60
		m = (seconds / 60) % 60
		h = seconds / 3600
	)

	if s == 0 {
		return fmt.Sprintf("%02d:%02d", h, m)
	} else {
		return fmt.Sprintf("%02d:%02d:%02d", h, m, s)
	}
}

func DayUTCOffsetSecs(t *time.Time) int {
	_, utcOffsetSeconds := t.Zone()
	return utcOffsetSeconds
}

func DayYYYYMMDD(t *time.Time) uint {
	y, m, d := t.Date()
	return uint(y*10000) + uint((m * 100)) + uint(d)
}

func DaySeconds(t *time.Time) uint {
	y, m, d := t.Date()
	midnight := time.Date(y, m, d, 0, 0, 0, 0, t.Location())
	return uint(t.Sub(midnight).Seconds())
}

func TimezoneToUTCDeltaSecs(yyyymmdd uint, timezone string) (int, error) {
	var (
		year  = int(yyyymmdd / 10000)
		month = int((yyyymmdd / 100) % 100)
		day   = int(yyyymmdd % 100)
	)
	if location, err := time.LoadLocation(timezone); err != nil {
		return 0, err
	} else {
		t := time.Date(year, time.Month(month), day, 0, 0, 0, 0, location)
		_, offset := t.Zone()
		return offset, nil
	}
}
